<?php


namespace System\Libraries;


/**
 * Load config files from /config folder in each environment that was set in the /index.php file.
 * This class can load config file, get config value, set config name and value (but not write to the file).
 *
 * @author Vee W.
 */
class Config
{


    protected $loaded_file = [];
    public $trace_loaded_files = [];


    /**
     * get the config value from specified name.
     * 
     * @param string $config_key the config name. this can set to ALL to get all the config values.
     * @param string $file config file name that were loaded. this is without extension.
     * @param string $default default value if config name is not found.
     * @return mixed return config values.
     */
    public function get($config_key, $file, $default = '')
    {
        $this->isLoaded($file);

        if (is_array($this->loaded_file) && array_key_exists($file, $this->loaded_file)) {
            if ($config_key != 'ALL' && is_array($this->loaded_file[$file]) && array_key_exists($config_key, $this->loaded_file[$file])) {
                return $this->loaded_file[$file][$config_key];
            } elseif ($config_key == 'ALL') {
                return $this->loaded_file[$file];
            } else {
                // config key not found.
            }
        } else {
            // config file could not be loaded.
        }

        return $default;
    }// get


    /**
     * check is config file loaded. if not, load the config file.
     * 
     * @param string $file the config file name without extension. case sensitive.
     * @return boolean
     */
    private function isLoaded($file)
    {
        if (!is_array($this->loaded_file) || (is_array($this->loaded_file) && !array_key_exists($file, $this->loaded_file))) {
            // config file did not load yet. load it.
            return $this->load($file);
        }

        return true;
    }// isLoaded


    /**
     * load the config file into property array.
     * 
     * @param string $file the config file name without extension. case sensitive.
     * @return boolean return true on loaded, false for otherwise.
     */
    public function load($file)
    {
        if (APP_ENV == 'production') {
            $config_env_dir = 'production';
        } elseif (APP_ENV == 'test') {
            $config_env_dir = 'test';
        } else {
            $config_env_dir = 'development';
        }

        if (file_exists(ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php') && is_file(ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php')) {
            // load config file from /config/ENV/.
            $this->loaded_file[$file] = require ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php';
            $this->trace_loaded_files[] = ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php';
            return true;
        } else {
            $envs = ['production', 'test', 'development'];
            foreach ($envs as $config_env_dir) {
                if (file_exists(ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php') && is_file(ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php')) {
                    // load config file from /config/ENV (by order in $envs)/.
                    $this->loaded_file[$file] = require ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php';
                    $this->trace_loaded_files[] = ROOT_PATH.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.$config_env_dir.DIRECTORY_SEPARATOR.$file.'.php';
                    unset($envs);
                    return true;
                }
            }
            unset($envs);
        }

        // config file could not found.
        return false;
    }// load


    /**
     * set config name (key) and value. warning! this will not write to the config file.
     * 
     * @param string $file config file name only, no extension.
     * @param string $config_key config name.
     * @param mixed $config_value config value.
     */
    public function set($file, $config_key, $config_value)
    {
        $this->isLoaded($file);

        if (is_array($this->loaded_file) && array_key_exists($file, $this->loaded_file)) {
            if (is_array($this->loaded_file[$file])) {
                $this->loaded_file[$file][$config_key] = $config_value;
            } else {
                // config file that were loaded is not an array.
            }
        } else {
            // config file could not be loaded.
        }
    }// set


}
